#!/bin/sh 
# \
exec wish "$0" "$@"

;# NAME:   getGET.tcl
;# AUTHOR: Clif Flynt
;# DATE:   Mar 5, 2002
;# DESC:   Extracts GET commands from the text session of an HTML
;#         Conversation.
;# 
;#         Use tcpdumpTEXT.tcl to extract HTTP transfers from tcpdump
;#         output.
;# 
;#         Use getGET.tcl to convert the text output into a tcl script.
;# 
;#         Invoke the testHarness with the name of that output and a
;#         status file identifier.
;# 
;#         tclsh tcpdumpTEXT.tcl <tcpdumpOutput | tclsh getGET.tcl >test.tcl
;#         wish testHarness.tcl test.tcl testID
;#
;#  This code was described in login magazine at www.usenix.org
;# 
;# PARAMETERS:  getGET.tcl ?fileName?
;#
;# RCSID: $Header: /home/cvs/WWWTest/getGET.tcl,v 1.1.1.1 2002/03/05 19:20:44 clif Exp $
;#
;# This code copyright 2002 to Clif Flynt
;# 9300 Fleming Rd
;# Dexter, MI  48130
;# clif@cflynt.com
;# All Rights Reserved.

set infl stdin

foreach {key val} $argv {
    set [string range $key 1 end] $val
}

if {![string match $infl stdin]} {
    set infl [open $infl r]
}

set fmt {set %s [http::geturl }
#set fmta {   http://$headerVal(Host)%s }
set fmta {   http://%s%s -headers $hdr }
set fmtb {   -query $query }
set fmtc {   -command "done %s %s [clock clicks]" ]}

set id 10

set prevInput ""

while {[set len [gets $infl line]] >= 0} {
    set line [string trim $line]
    if {[string length $line] == 0} {continue}

    if {([string first GET $line] == 0) ||
        ([string first POST $line] == 0)} {

      if {[string match $prevInput ""]} {
          lappend prevInput $line
          continue
	  }
	  
      set testLine [lindex $prevInput 0]
      
      foreach index {Cookie:  Host: } {
	  set i [string trim $index :]
          set p [lsearch $prevInput $index*]
	  if {$p < 0}  {
	      catch unset headerVal($i)
	      continue 
	  }
	  set l [lindex $prevInput $p]
	  set headerVal($i) [string range $l [expr [string length $index] + 1] end]
      }
      
      if {[info exists headerVal(Cookie)]} {
          set hdrs "Cookie {$headerVal(Cookie)}"
      } else {
          set hdrs ""
      }
      
      # Process a GET
      if {[string first GET $testLine] == 0} {
           set site [string range $testLine [string first " " $testLine] [string last " " $testLine]]
           set site [string trim $site]

#	   puts "set query $query\n"
	   puts "set hdr {$hdrs}\n"
	   puts ".l configure -text $site"
           puts "global  WaitVar token$id"
           puts "set  WaitVar($id) 0"
	   puts "update idle"

	   puts [format $fmt token$id]\\
	   puts [format $fmta  $headerVal(Host) $site]\\
#	   puts [format $fmtb  ]\\
	   puts [format $fmtc  $site $id]
	   puts ""
           puts "vwait WaitVar($id)"
	   puts "\n"

	   incr id
       }

      # Process a POST
       if {([string first POST $testLine] == 0)} {
           # Implementing POST left as an exercise for the reader.
	   # (Sorry, I didn't need it for my test, and didn't get
	   #  around to filling in the space.)
       }

         set prevInput ""
	 lappend prevInput $line
     } else {
         lappend prevInput $line
     }
}

puts ".l configure -text {TEST COMPLETE}"

puts "checkResults"
