#!/bin/sh 
# \
exec wish "$0" "$@"

;# NAME:   tcpdumpTEXT.tcl
;# AUTHOR: Clif Flynt
;# DATE:   Mar 5, 2002
;# DESC:   Extracts TEXT from tcpdump output.
;#
;#  This code was described in login magazine at www.usenix.org
;# 
;# PARAMETERS:  tcpdumpTEXT.tcl <tcpdumpOutput
;#
;# RCSID: $Header: /home/cvs/WWWTest/tcpdumpTEXT.tcl,v 1.1.1.1 2002/03/05 19:20:44 clif Exp $
;#
;# This code copyright 2002 to Clif Flynt
;# 9300 Fleming Rd
;# Dexter, MI  48130
;# clif@cflynt.com
;# All Rights Reserved.

################################################################
# proc hex2Text {hex}--
#    Convert tcpdump formatted hex output to text
#    Skips TCP and IP headers
# Arguments
#   hex		The hex output from tcpdump
# 
# Results
#   Returns a string of (hopefully) printable ASCII
# 
proc hex2Text {hex} {
    set hex [join $hex ""]

    foreach {h l} [split $hex ""] {
        lappend bytes $h$l
    }

    # The IP header length is the second nibble.
    # The TCP header length is the first nibble of the 13'th byte
    # Both are in terms of 4 byte words

    set headerLen [string range [lindex $bytes 0] 1 1]
    set IPhdrBytes [expr $headerLen * 4]

    set TcpHdrLen [string range [lindex $bytes [expr $IPhdrBytes + 12]] 0 0]
    set TcpHdrBytes [expr $TcpHdrLen * 4]
    
    #     puts "TCPHDRLEN: $TcpHdrLen - $TcpHdrBytes"

    set pos [expr $IPhdrBytes + $TcpHdrBytes]

    set str ""
    foreach b [lrange $bytes $pos end] {
         append str [format %c 0x$b]
    }
    return "$str"
}

################################################################
# proc getArgs {}--
#    Extract arguments from argv
# Arguments
#   NONE
# 
# Results
#   Creates a bunch of global variables based on the command line
#
#   NOTE: This was a noble experiment, but I don't really like the
#   technique of having many variables.  I think it's better to use
#   the global state array with many indices.
# 
proc getArgs {} {
  global argv0
  global argv 
  
  # Define the options/varNames and default values

  set -packetLen       100
  set -flagPattern     "P"
  set -destIPpattern   "*"
  set -srcIPpattern    "*"
  set -srcPORTpattern  "*"
  set -destPORTpattern "http:"
  set -input            stdin

  # Initialize a failure message in case of an illegal option.

  set failMsg "Usage: $argv0 ?-flag value? ?inputFile?"
  append failMsg "\nOptions (and defaults) are:\n"
  
  # Set global variables, and expand the failure message.
  foreach l [info locals -*] {
    set l1 [string range $l 1 end]
    global $l1
    set $l1 [set $l]
    append failMsg "\n   $l ([set $l])"
  }

  append failMsg "\n\nTCPDUMP CMD: tcpdump -s 15000 -l -x -n -v -i ethx"
  
  if {[llength $argv] & 1} {
      set input [lindex $argv end]
      # "end-1" works for tclsh 8.1 and newer.
      # for older tclsh use [expr [llength $argv] -2]
      set argv [lrange $argv 0 end-1]
  }

  foreach {key val} $argv {
    if {![info exists $key]} {
        puts $failMsg
	exit
    }
    set varName [string range $key 1 end] 
    set $varName $val
  }
  
  if {![string match $input stdin]} {
    if {[catch {open $input r} input]} {
        puts $failMsg
	exit
    }
  }
}

getArgs

set hex ""
set oldLine ""
set TcpFlag ""
set destPORT ""

set addr {([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)}
set port {.([^ ]+)}
set sep {[ <>]+}
set flag {([^ ])}

while {[set len [gets $input line]] >= 0} {
  if {$len < 2} continue

  # A colon in position 2 means a header line xx:yy:zz.abc
  if {[string first ":" $line] == 2} {

     set m [regexp "$addr$port$sep$addr$port +$flag" \
        $oldLine all srcIP srcPORT destIP destPORT TcpFlag]

     if {([string length $hex] > $packetLen) &&
        ([string match 4 [string range [string trimleft $hex] 0 0]]) &&
        ([string match $destIPpattern $destIP]) && 
        ([string match $destPORTpattern $destPORT ]) && 
        ([string match $srcIPpattern $srcIP]) && 
        ([string match $srcPORTpattern $srcPORT ]) && 
        ([string match $flagPattern $TcpFlag ])} {
         set txt [hex2Text $hex]
         puts "\n$txt\n-----------------------\n";
	 }

    set oldLine $line
    set hex ""; 
  } else {
     append hex "[string trim $line]"
  }
}
